@@ -43,62 +43,133 @@ class MemberFlows(object):
                       constants.LISTENERS,
                       constants.LOADBALANCER,
                       constants.POOL]))
+        create_member_flow.add(vthunder_tasks.VthunderInstanceBusy(
+            requires=a10constants.COMPUTE_BUSY))
+
         create_member_flow.add(database_tasks.MarkMemberPendingCreateInDB(
             requires=constants.MEMBER))
-        create_member_flow.add(a10_network_tasks.CalculateDelta(
-            requires=constants.LOADBALANCER,
-            provides=constants.DELTAS))
-        create_member_flow.add(a10_network_tasks.HandleNetworkDeltas(
-            requires=constants.DELTAS, provides=constants.ADDED_PORTS))
-        create_member_flow.add(database_tasks.GetAmphoraeFromLoadbalancer(
-            requires=constants.LOADBALANCER,
-            provides=constants.AMPHORA))
         create_member_flow.add(a10_database_tasks.GetVThunderByLoadBalancer(
             requires=constants.LOADBALANCER,
             provides=a10constants.VTHUNDER))
+        create_member_flow.add(a10_database_tasks.GetLoadBalancerListByProjectID(
+            requires=a10constants.VTHUNDER,
+            provides=a10constants.LOADBALANCERS_LIST))
+        create_member_flow.add(database_tasks.GetAmphoraeFromLoadbalancer(
+            requires=constants.LOADBALANCER,
+            provides=constants.AMPHORA))
+        create_member_flow.add(a10_network_tasks.CalculateDelta(
+            requires=(constants.LOADBALANCER, a10constants.LOADBALANCERS_LIST),
+            provides=constants.DELTAS))
+        create_member_flow.add(a10_network_tasks.HandleNetworkDeltas(
+            requires=constants.DELTAS, provides=constants.ADDED_PORTS))
         # managing interface additions here
-        create_member_flow.add(
-            vthunder_tasks.AmphoraePostMemberNetworkPlug(
-                requires=(
-                    constants.LOADBALANCER,
-                    constants.ADDED_PORTS,
-                    a10constants.VTHUNDER)))
-        create_member_flow.add(vthunder_tasks.VThunderComputeConnectivityWait(
-            requires=(a10constants.VTHUNDER, constants.AMPHORA)))
-        create_member_flow.add(
-            vthunder_tasks.EnableInterfaceForMembers(
-                requires=[
-                    constants.ADDED_PORTS,
-                    constants.LOADBALANCER,
-                    a10constants.VTHUNDER]))
+        if topology == constants.TOPOLOGY_SINGLE:
+            create_member_flow.add(
+                vthunder_tasks.AmphoraePostMemberNetworkPlug(
+                    requires=(
+                        constants.LOADBALANCER,
+                        constants.ADDED_PORTS,
+                        a10constants.VTHUNDER)))
+            create_member_flow.add(vthunder_tasks.VThunderComputeConnectivityWait(
+                name=a10constants.VTHUNDER_CONNECTIVITY_WAIT,
+                requires=(a10constants.VTHUNDER, constants.AMPHORA)))
+            create_member_flow.add(
+                vthunder_tasks.EnableInterfaceForMembers(
+                    requires=[
+                        constants.ADDED_PORTS,
+                        constants.LOADBALANCER,
+                        a10constants.VTHUNDER]))
         # configure member flow for HA
         if topology == constants.TOPOLOGY_ACTIVE_STANDBY:
             create_member_flow.add(
                 a10_database_tasks.GetBackupVThunderByLoadBalancer(
                     name="get_backup_vThunder",
-                    requires=constants.LOADBALANCER,
+                    requires=(constants.LOADBALANCER, a10constants.VTHUNDER),
                     provides=a10constants.BACKUP_VTHUNDER))
+            create_member_flow.add(vthunder_tasks.VThunderComputeConnectivityWait(
+                name="backup_compute_conn_wait_before_probe_device",
+                requires=constants.AMPHORA,
+                rebind={a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
+            create_member_flow.add(vthunder_tasks.VCSSyncWait(
+                name="vcs_sync_wait_before_probe_device",
+                requires=a10constants.VTHUNDER))
+            create_member_flow.add(
+                vthunder_tasks.AmphoraePostMemberNetworkPlug(
+                    requires=(
+                        constants.LOADBALANCER,
+                        constants.ADDED_PORTS,
+                        a10constants.VTHUNDER)))
             create_member_flow.add(
                 vthunder_tasks.AmphoraePostMemberNetworkPlug(
                     name="backup_amphora_network_plug", requires=[
                         constants.ADDED_PORTS, constants.LOADBALANCER], rebind={
                         a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
-            create_member_flow.add(
-                vthunder_tasks.VThunderComputeConnectivityWait(
-                    name="backup_compute_conn_wait", requires=constants.AMPHORA, rebind={
-                        a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
+            create_member_flow.add(vthunder_tasks.VThunderComputeConnectivityWait(
+                name=a10constants.MASTER_CONNECTIVITY_WAIT,
+                requires=(a10constants.VTHUNDER, constants.AMPHORA)))
+            create_member_flow.add(vthunder_tasks.VThunderComputeConnectivityWait(
+                name=a10constants.BACKUP_CONNECTIVITY_WAIT,
+                requires=constants.AMPHORA,
+                rebind={a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
+            create_member_flow.add(vthunder_tasks.VCSSyncWait(
+                name="backup-plug-wait-vcs-ready",
+                requires=a10constants.VTHUNDER))
+            create_member_flow.add(vthunder_tasks.GetMasterVThunder(
+                name=a10constants.GET_MASTER_VTHUNDER,
+                requires=a10constants.VTHUNDER,
+                provides=a10constants.VTHUNDER))
+            create_member_flow.add(a10_database_tasks.GetBackupVThunderByLoadBalancer(
+                name="get_backup_vThunder_after_get_master",
+                requires=(constants.LOADBALANCER, a10constants.VTHUNDER),
+                provides=a10constants.BACKUP_VTHUNDER))
             create_member_flow.add(
                 vthunder_tasks.EnableInterfaceForMembers(
-                    name="backup_enable_interface", requires=[
-                        constants.ADDED_PORTS, constants.LOADBALANCER], rebind={
-                        a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
-
+                    name=a10constants.ENABLE_MASTER_VTHUNDER_INTERFACE,
+                    requires=[
+                        constants.ADDED_PORTS,
+                        constants.LOADBALANCER,
+                        a10constants.VTHUNDER]))
+            create_member_flow.add(vthunder_tasks.AmphoraePostMemberNetworkPlug(
+                name="amphorae-post-member-network-plug-for-master",
+                requires=(constants.LOADBALANCER, constants.ADDED_PORTS,
+                          a10constants.VTHUNDER)))
+            create_member_flow.add(
+                vthunder_tasks.VThunderComputeConnectivityWait(
+                    name=a10constants.CONNECTIVITY_WAIT_FOR_MASTER_VTHUNDER,
+                    requires=(a10constants.VTHUNDER, constants.AMPHORA)))
+            create_member_flow.add(
+                vthunder_tasks.VThunderComputeConnectivityWait(
+                    name=a10constants.CONNECTIVITY_WAIT_FOR_BACKUP_VTHUNDER,
+                    rebind={
+                        a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER},
+                    requires=constants.AMPHORA))
+            create_member_flow.add(vthunder_tasks.VCSSyncWait(
+                name="member_enable_interface_vcs_sync_wait",
+                requires=a10constants.VTHUNDER))
+            create_member_flow.add(vthunder_tasks.GetMasterVThunder(
+                name=a10constants.GET_VTHUNDER_MASTER,
+                requires=a10constants.VTHUNDER,
+                provides=a10constants.VTHUNDER))
+            create_member_flow.add(
+                vthunder_tasks.EnableInterfaceForMembers(
+                    requires=[
+                        constants.ADDED_PORTS,
+                        constants.LOADBALANCER,
+                        a10constants.VTHUNDER]))
+        create_member_flow.add(self.handle_vrid_for_member_subflow())
         create_member_flow.add(a10_database_tasks.CountMembersWithIP(
             requires=constants.MEMBER, provides=a10constants.MEMBER_COUNT_IP
         ))
+        create_member_flow.add(vthunder_tasks.AllowLoadbalancerForwardWithAnySource(
+            name=a10constants.ALLOW_NO_SNAT,
+            requires=(constants.MEMBER, constants.AMPHORA)))
+        create_member_flow.add(a10_database_tasks.GetFlavorData(
+            rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+            provides=constants.FLAVOR))
+        create_member_flow.add(self.get_create_member_snat_pool_subflow())
         create_member_flow.add(server_tasks.MemberCreate(
             requires=(constants.MEMBER, a10constants.VTHUNDER, constants.POOL,
-                      a10constants.MEMBER_COUNT_IP)))
+                      a10constants.MEMBER_COUNT_IP, constants.FLAVOR)))
         create_member_flow.add(database_tasks.MarkMemberActiveInDB(
             requires=constants.MEMBER))
         create_member_flow.add(database_tasks.MarkPoolActiveInDB(
@@ -113,7 +184,7 @@ class MemberFlows(object):
             requires=a10constants.VTHUNDER))
         return create_member_flow
 
-    def get_delete_member_flow(self):
+    def get_delete_member_flow(self, topology):
         """Flow to delete a member on VThunder
 
         :returns: The flow for deleting a member
@@ -124,11 +195,17 @@ class MemberFlows(object):
                       constants.LISTENERS,
                       constants.LOADBALANCER,
                       constants.POOL]))
+        delete_member_flow.add(vthunder_tasks.VthunderInstanceBusy(
+            requires=a10constants.COMPUTE_BUSY))
+
         delete_member_flow.add(database_tasks.MarkMemberPendingDeleteInDB(
             requires=constants.MEMBER))
         delete_member_flow.add(model_tasks.
                                DeleteModelObject(rebind={constants.OBJECT:
                                                          constants.MEMBER}))
+        delete_member_flow.add(database_tasks.GetAmphoraeFromLoadbalancer(
+            requires=constants.LOADBALANCER,
+            provides=constants.AMPHORA))
         delete_member_flow.add(a10_database_tasks.GetVThunderByLoadBalancer(
             requires=constants.LOADBALANCER,
             provides=a10constants.VTHUNDER))
@@ -143,6 +220,63 @@ class MemberFlows(object):
         delete_member_flow.add(a10_database_tasks.GetFlavorData(
             rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
             provides=constants.FLAVOR))
+        delete_member_flow.add(database_tasks.DeleteMemberInDB(
+            requires=constants.MEMBER))
+        delete_member_flow.add(a10_database_tasks.GetLoadBalancerListByProjectID(
+            requires=a10constants.VTHUNDER,
+            provides=a10constants.LOADBALANCERS_LIST))
+        delete_member_flow.add(a10_network_tasks.CalculateDelta(
+            requires=(constants.LOADBALANCER, a10constants.LOADBALANCERS_LIST),
+            provides=constants.DELTAS))
+        delete_member_flow.add(a10_network_tasks.HandleNetworkDeltas(
+            requires=constants.DELTAS, provides=constants.ADDED_PORTS))
+        delete_member_flow.add(
+            vthunder_tasks.AmphoraePostNetworkUnplug(
+                requires=(
+                    constants.LOADBALANCER,
+                    constants.ADDED_PORTS,
+                    a10constants.VTHUNDER)))
+        delete_member_flow.add(vthunder_tasks.VThunderComputeConnectivityWait(
+            name=a10constants.VTHUNDER_CONNECTIVITY_WAIT,
+            requires=(a10constants.VTHUNDER, constants.AMPHORA)))
+        if topology == constants.TOPOLOGY_ACTIVE_STANDBY:
+            delete_member_flow.add(
+                a10_database_tasks.GetBackupVThunderByLoadBalancer(
+                    name=a10constants.GET_BACKUP_VTHUNDER_BY_LB,
+                    requires=(constants.LOADBALANCER, a10constants.VTHUNDER),
+                    provides=a10constants.BACKUP_VTHUNDER))
+            delete_member_flow.add(
+                vthunder_tasks.VThunderComputeConnectivityWait(
+                    name=a10constants.BACKUP_CONNECTIVITY_WAIT + "-before-unplug",
+                    requires=constants.AMPHORA,
+                    rebind={a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
+            delete_member_flow.add(
+                vthunder_tasks.AmphoraePostNetworkUnplug(
+                    name=a10constants.AMPHORA_POST_NETWORK_UNPLUG_FOR_BACKUP_VTHUNDER,
+                    requires=(constants.LOADBALANCER, constants.ADDED_PORTS),
+                    rebind={a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
+            delete_member_flow.add(
+                vthunder_tasks.VThunderComputeConnectivityWait(
+                    name=a10constants.BACKUP_CONNECTIVITY_WAIT,
+                    requires=constants.AMPHORA,
+                    rebind={a10constants.VTHUNDER: a10constants.BACKUP_VTHUNDER}))
+            delete_member_flow.add(vthunder_tasks.VCSSyncWait(
+                name='member-unplug-' + a10constants.VCS_SYNC_WAIT,
+                requires=a10constants.VTHUNDER))
+            delete_member_flow.add(vthunder_tasks.GetMasterVThunder(
+                name=a10constants.GET_VTHUNDER_MASTER,
+                requires=a10constants.VTHUNDER,
+                provides=a10constants.VTHUNDER))
+        delete_member_flow.add(server_tasks.MemberFindNatPool(
+            requires=[constants.MEMBER, a10constants.VTHUNDER, constants.POOL,
+                      constants.FLAVOR], provides=a10constants.NAT_FLAVOR))
+        delete_member_flow.add(a10_database_tasks.GetNatPoolEntry(
+            requires=[constants.MEMBER, a10constants.NAT_FLAVOR],
+            provides=a10constants.NAT_POOL))
+        delete_member_flow.add(a10_network_tasks.ReleaseSubnetAddressForMember(
+            requires=[constants.MEMBER, a10constants.NAT_FLAVOR, a10constants.NAT_POOL]))
+        delete_member_flow.add(a10_database_tasks.DeleteNatPoolEntry(
+            requires=a10constants.NAT_POOL))
         delete_member_flow.add(
             server_tasks.MemberDelete(
                 requires=(
@@ -151,8 +285,7 @@ class MemberFlows(object):
                     constants.POOL,
                     a10constants.MEMBER_COUNT_IP,
                     a10constants.MEMBER_COUNT_IP_PORT_PROTOCOL)))
-        delete_member_flow.add(database_tasks.DeleteMemberInDB(
-            requires=constants.MEMBER))
+        delete_member_flow.add(self.get_delete_member_vrid_subflow())
         delete_member_flow.add(database_tasks.DecrementMemberQuota(
             requires=constants.MEMBER))
         delete_member_flow.add(database_tasks.MarkPoolActiveInDB(
@@ -239,13 +372,9 @@ class MemberFlows(object):
             requires=a10constants.VTHUNDER))
         return delete_member_flow
 
-    def get_delete_member_vthunder_internal_subflow(self, member_id):
+    def get_delete_member_vthunder_internal_subflow(self, member_id, pool):
         delete_member_thunder_subflow = linear_flow.Flow(
             a10constants.DELETE_MEMBER_VTHUNDER_INTERNAL_SUBFLOW)
-        delete_member_thunder_subflow.add(vthunder_tasks.SetupDeviceNetworkMap(
-            name='setup_device_network_map_' + member_id,
-            requires=a10constants.VTHUNDER,
-            provides=a10constants.VTHUNDER))
         delete_member_thunder_subflow.add(
             a10_database_tasks.CountMembersWithIPPortProtocol(
                 name='count_members_ip_port_' + member_id,
@@ -254,7 +383,8 @@ class MemberFlows(object):
                     constants.POOL),
                 provides=a10constants.MEMBER_COUNT_IP_PORT_PROTOCOL,
                 rebind={
-                    constants.MEMBER: member_id}))
+                    constants.MEMBER: member_id,
+                    constants.POOL: pool}))
         delete_member_thunder_subflow.add(a10_database_tasks.PoolCountforIP(
             name='pool_count_for_ip_' + member_id,
             requires=constants.MEMBER, provides=a10constants.POOL_COUNT_IP,
@@ -269,7 +399,7 @@ class MemberFlows(object):
             name='member_find_nat_pool_' + member_id,
             requires=[constants.MEMBER, a10constants.VTHUNDER, constants.POOL,
                       constants.FLAVOR], provides=a10constants.NAT_FLAVOR,
-            rebind={constants.MEMBER: member_id}))
+            rebind={constants.MEMBER: member_id, constants.POOL: pool}))
         delete_member_thunder_subflow.add(a10_database_tasks.GetNatPoolEntry(
             name='get_nat_pool_db_entry_' + member_id,
             requires=[constants.MEMBER, a10constants.NAT_FLAVOR],
@@ -293,7 +423,7 @@ class MemberFlows(object):
                     a10constants.POOL_COUNT_IP,
                     a10constants.MEMBER_COUNT_IP_PORT_PROTOCOL),
                 rebind={
-                    constants.MEMBER: member_id}))
+                    constants.MEMBER: member_id, constants.POOL: pool}))
         if CONF.a10_global.network_type == 'vlan':
             delete_member_thunder_subflow.add(
                 vthunder_tasks.DeleteInterfaceTagIfNotInUseForMember(
@@ -321,7 +451,7 @@ class MemberFlows(object):
         delete_member_vrid_subflow.add(
             a10_database_tasks.CountLoadbalancersInProjectBySubnet(
                 requires=[constants.SUBNET, a10constants.PARTITION_PROJECT_LIST],
-                provides=a10constants.LB_COUNT))
+                provides=a10constants.LB_COUNT_SUBNET))
         delete_member_vrid_subflow.add(
             a10_database_tasks.CountMembersInProjectBySubnet(
                 requires=[constants.SUBNET, a10constants.PARTITION_PROJECT_LIST],
@@ -336,8 +466,9 @@ class MemberFlows(object):
                     a10constants.VTHUNDER,
                     a10constants.VRID_LIST,
                     constants.SUBNET,
-                    a10constants.LB_COUNT,
+                    a10constants.LB_COUNT_SUBNET,
                     a10constants.MEMBER_COUNT],
+                rebind={a10constants.LB_RESOURCE: constants.MEMBER},
                 provides=(
                     a10constants.VRID,
                     a10constants.DELETE_VRID)))
@@ -345,30 +476,37 @@ class MemberFlows(object):
             requires=[a10constants.VRID, a10constants.DELETE_VRID]))
         return delete_member_vrid_subflow
 
-    def get_delete_member_vrid_internal_subflow(self):
+    def get_delete_member_vrid_internal_subflow(self, pool, pool_members):
         delete_member_vrid_subflow = linear_flow.Flow(
             a10constants.DELETE_MEMBER_VRID_INTERNAL_SUBFLOW)
         delete_member_vrid_subflow.add(
             a10_database_tasks.GetChildProjectsOfParentPartition(
-                rebind={a10constants.LB_RESOURCE: constants.POOL},
+                name='get_child_project_of_parent_partition' + pool,
+                rebind={a10constants.LB_RESOURCE: pool},
                 provides=a10constants.PARTITION_PROJECT_LIST
             ))
         delete_member_vrid_subflow.add(
             a10_database_tasks.GetSubnetForDeletionInPool(
+                name='get_subnet_for_deletion_in_pool' + pool,
                 requires=[a10constants.MEMBER_LIST, a10constants.PARTITION_PROJECT_LIST],
+                rebind={a10constants.MEMBER_LIST: pool_members},
                 provides=a10constants.SUBNET_LIST))
         delete_member_vrid_subflow.add(
             a10_database_tasks.GetVRIDForLoadbalancerResource(
+                name='get_vrid_for_loadbalancer_resource' + pool,
                 requires=a10constants.PARTITION_PROJECT_LIST,
                 provides=a10constants.VRID_LIST))
         delete_member_vrid_subflow.add(
             a10_network_tasks.DeleteMultipleVRIDPort(
+                name='delete_multiple_vrid_port' + pool,
                 requires=[
                     a10constants.VTHUNDER,
                     a10constants.VRID_LIST,
                     a10constants.SUBNET_LIST],
+                rebind={a10constants.LB_RESOURCE: pool},
                 provides=a10constants.VRID_LIST))
         delete_member_vrid_subflow.add(a10_database_tasks.DeleteMultiVRIDEntry(
+            name='delete_multi_vrid_entry' + pool,
             requires=a10constants.VRID_LIST))
         return delete_member_vrid_subflow
 
@@ -406,7 +544,7 @@ class MemberFlows(object):
 
         return handle_vrid_for_member_subflow
 
-    def get_update_member_flow(self):
+    def get_update_member_flow(self, topology):
         """Flow to update a member
 
         :returns: The flow for updating a member
@@ -417,13 +555,26 @@ class MemberFlows(object):
                       constants.LISTENERS,
                       constants.LOADBALANCER,
                       constants.POOL]))
+        update_member_flow.add(vthunder_tasks.VthunderInstanceBusy(
+            requires=a10constants.COMPUTE_BUSY))
+
         update_member_flow.add(database_tasks.MarkMemberPendingUpdateInDB(
             requires=constants.MEMBER))
         update_member_flow.add(a10_database_tasks.GetVThunderByLoadBalancer(
             requires=constants.LOADBALANCER,
             provides=a10constants.VTHUNDER))
+        if topology == constants.TOPOLOGY_ACTIVE_STANDBY:
+            update_member_flow.add(vthunder_tasks.GetMasterVThunder(
+                name=a10constants.GET_MASTER_VTHUNDER,
+                requires=a10constants.VTHUNDER,
+                provides=a10constants.VTHUNDER))
+        update_member_flow.add(self.handle_vrid_for_member_subflow())
+        update_member_flow.add(a10_database_tasks.GetFlavorData(
+            rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
+            provides=constants.FLAVOR))
         update_member_flow.add(server_tasks.MemberUpdate(
-            requires=(constants.MEMBER, a10constants.VTHUNDER, constants.POOL)))
+            requires=(constants.MEMBER, a10constants.VTHUNDER,
+                      constants.POOL, constants.FLAVOR)))
         update_member_flow.add(database_tasks.UpdateMemberInDB(
             requires=[constants.MEMBER, constants.UPDATE_DICT]))
         update_member_flow.add(database_tasks.MarkMemberActiveInDB(
@@ -515,18 +666,7 @@ class MemberFlows(object):
         create_member_flow.add(a10_database_tasks.GetFlavorData(
             rebind={a10constants.LB_RESOURCE: constants.LOADBALANCER},
             provides=constants.FLAVOR))
-        create_member_flow.add(server_tasks.MemberFindNatPool(
-            requires=[constants.MEMBER, a10constants.VTHUNDER, constants.POOL,
-                      constants.FLAVOR], provides=a10constants.NAT_FLAVOR))
-        create_member_flow.add(a10_database_tasks.GetNatPoolEntry(
-            requires=[constants.MEMBER, a10constants.NAT_FLAVOR],
-            provides=a10constants.NAT_POOL))
-        create_member_flow.add(a10_network_tasks.ReserveSubnetAddressForMember(
-            requires=[constants.MEMBER, a10constants.NAT_FLAVOR, a10constants.NAT_POOL],
-            provides=a10constants.SUBNET_PORT))
-        create_member_flow.add(a10_database_tasks.UpdateNatPoolDB(
-            requires=[constants.MEMBER, a10constants.NAT_FLAVOR,
-                      a10constants.NAT_POOL, a10constants.SUBNET_PORT]))
+        create_member_flow.add(self.get_create_member_snat_pool_subflow())
         create_member_flow.add(server_tasks.MemberCreate(
             requires=(constants.MEMBER, a10constants.VTHUNDER, constants.POOL,
                       a10constants.MEMBER_COUNT_IP, constants.FLAVOR)))
@@ -543,3 +683,20 @@ class MemberFlows(object):
         create_member_flow.add(a10_database_tasks.SetThunderUpdatedAt(
             requires=a10constants.VTHUNDER))
         return create_member_flow
+
+    def get_create_member_snat_pool_subflow(self):
+        create_member_snat_subflow = linear_flow.Flow(
+            a10constants.CREATE_MEMBER_SNAT_POOL_SUBFLOW)
+        create_member_snat_subflow.add(server_tasks.MemberFindNatPool(
+            requires=[constants.MEMBER, a10constants.VTHUNDER, constants.POOL,
+                      constants.FLAVOR], provides=a10constants.NAT_FLAVOR))
+        create_member_snat_subflow.add(a10_database_tasks.GetNatPoolEntry(
+            requires=[constants.MEMBER, a10constants.NAT_FLAVOR],
+            provides=a10constants.NAT_POOL))
+        create_member_snat_subflow.add(a10_network_tasks.ReserveSubnetAddressForMember(
+            requires=[constants.MEMBER, a10constants.NAT_FLAVOR, a10constants.NAT_POOL],
+            provides=a10constants.SUBNET_PORT))
+        create_member_snat_subflow.add(a10_database_tasks.UpdateNatPoolDB(
+            requires=[constants.MEMBER, a10constants.NAT_FLAVOR,
+                      a10constants.NAT_POOL, a10constants.SUBNET_PORT]))
+        return create_member_snat_subflow
