@@ -21,6 +21,9 @@ from taskflow.types import failure
 from octavia.common import constants
 from octavia.common import exceptions
 from octavia.controller.worker.tasks.compute_tasks import BaseComputeTask
+from octavia.db import api as db_apis
+
+from a10_octavia.db import repositories as a10_repo
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -37,7 +40,8 @@ class ComputeCreate(BaseComputeTask):
         network_ids = CONF.a10_controller_worker.amp_boot_network_list[:]
         # Injecting VIP network ID
         if loadbalancer:
-            network_ids.append(loadbalancer.vip.network_id)
+            if loadbalancer.vip.network_id not in network_ids:
+                network_ids.append(loadbalancer.vip.network_id)
         LOG.debug("Compute create execute for amphora with id %s", amphora_id)
         key_name = CONF.a10_controller_worker.amp_ssh_key_name
 
@@ -83,6 +87,29 @@ class ComputeCreate(BaseComputeTask):
                           compute_id, str(e))
 
 
+class ValidateComputeForProject(BaseComputeTask):
+    """Validate compute_id got for the project"""
+
+    def execute(self, loadbalancer, role):
+        self.vthunder_repo = a10_repo.VThunderRepository()
+        vthunder_ids = self.vthunder_repo.get_vthunders_by_project_id_and_role(
+            db_apis.get_session(), loadbalancer.project_id, role)
+        for vthunder_id in vthunder_ids:
+            vthunder = self.vthunder_repo.get(
+                db_apis.get_session(),
+                id=vthunder_id)
+            try:
+                amp, fault = self.compute.get_amphora(vthunder.compute_id)
+                if amp.compute_id == vthunder.compute_id:
+                    LOG.debug("Successfully validated comput_id %s for the project %s",
+                              vthunder.compute_id, vthunder.project_id)
+                    break
+            except Exception:
+                pass
+
+        return vthunder.compute_id
+
+
 class ComputeActiveWait(BaseComputeTask):
     """Wait for the compute driver to mark the amphora active."""
 
