@@ -13,10 +13,10 @@
 # under the License.
 #
 import acos_client.errors as acos_errors
-import copy
 from neutronclient.common import exceptions as neutron_exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import uuidutils
 from requests import exceptions as req_exceptions
 import six
 import socket
@@ -33,7 +33,7 @@ from a10_octavia.common import a10constants
 from a10_octavia.common import data_models
 from a10_octavia.common import utils as a10_utils
 from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator
-
+from a10_octavia.controller.worker.tasks import utils as a10_task_utils
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
@@ -58,25 +58,34 @@ class CalculateAmphoraDelta(BaseNetworkTask):
 
     default_provides = constants.DELTA
 
-    def execute(self, loadbalancer, amphora):
+    def execute(self, loadbalancers_list, amphora):
         LOG.debug("Calculating network delta for amphora id: %s", amphora.id)
         # Figure out what networks we want
         # seed with lb network(s)
-        vrrp_port = self.network_driver.get_port(amphora.vrrp_port_id)
-        desired_network_ids = {vrrp_port.network_id}.union(
-            CONF.controller_worker.amp_boot_network_list)
 
-        for pool in loadbalancer.pools:
-            member_networks = [
-                self.network_driver.get_subnet(member.subnet_id).network_id
-                for member in pool.members
-                if member.subnet_id
-            ]
-            desired_network_ids.update(member_networks)
+        desired_network_ids = set(CONF.a10_controller_worker.amp_boot_network_list[:])
+        member_networks = []
+        for loadbalancer in loadbalancers_list:
+            for pool in loadbalancer.pools:
+                member_networks = [
+                    self.network_driver.get_subnet(member.subnet_id).network_id
+                    for member in pool.members
+                    if member.subnet_id
+                ]
+                desired_network_ids.update(member_networks)
+
+        loadbalancer_networks = [
+            self.network_driver.get_subnet(loadbalancer.vip.subnet_id).network_id
+            for loadbalancer in loadbalancers_list
+            if loadbalancer.vip.subnet_id
+        ]
+        desired_network_ids.update(loadbalancer_networks)
+        LOG.debug("[NetIF] desired_network_ids.update{0}".format(desired_network_ids))
 
         nics = self.network_driver.get_plugged_networks(amphora.compute_id)
         # assume we don't have two nics in the same network
         actual_network_nics = dict((nic.network_id, nic) for nic in nics)
+        LOG.debug("[NetIF] actual_network_nics {0}".format(actual_network_nics))
 
         del_ids = set(actual_network_nics) - desired_network_ids
         delete_nics = list(
@@ -101,7 +110,7 @@ class CalculateDelta(BaseNetworkTask):
 
     default_provides = constants.DELTAS
 
-    def execute(self, loadbalancer):
+    def execute(self, loadbalancer, loadbalancers_list):
         """Compute which NICs need to be plugged
 
         for the amphora to become operational.
@@ -118,7 +127,7 @@ class CalculateDelta(BaseNetworkTask):
             lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
                 loadbalancer.amphorae):
 
-            delta = calculate_amp.execute(loadbalancer, amphora)
+            delta = calculate_amp.execute(loadbalancers_list, amphora)
             deltas[amphora.id] = delta
         return deltas
 
@@ -246,6 +255,7 @@ class HandleNetworkDelta(BaseNetworkTask):
             try:
                 self.network_driver.unplug_network(delta.compute_id,
                                                    nic.network_id)
+
             except base.NetworkNotFound:
                 LOG.debug("Network %d not found ", nic.network_id)
             except Exception:
@@ -285,6 +295,7 @@ class HandleNetworkDeltas(BaseNetworkTask):
         for amp_id, delta in six.iteritems(deltas):
             added_ports[amp_id] = []
             for nic in delta.add_nics:
+                LOG.debug("[NetIF] plug_network %s on %s", nic.network_id, delta.compute_id)
                 interface = self.network_driver.plug_network(delta.compute_id,
                                                              nic.network_id)
                 port = self.network_driver.get_port(interface.port_id)
@@ -295,8 +306,11 @@ class HandleNetworkDeltas(BaseNetworkTask):
                 added_ports[amp_id].append(port)
             for nic in delta.delete_nics:
                 try:
+                    LOG.debug("[NetIF] unplug_network %s on %s", nic.network_id, delta.compute_id)
                     self.network_driver.unplug_network(delta.compute_id,
                                                        nic.network_id)
+                    network = self.network_driver.get_network(nic.network_id)
+                    added_ports[amp_id].append(network)
                 except base.NetworkNotFound:
                     LOG.debug("Network %d not found ", nic.network_id)
                 except Exception as e:
@@ -432,7 +446,7 @@ class UnplugVIP(BaseNetworkTask):
 class AllocateVIP(BaseNetworkTask):
     """Task to allocate a VIP."""
 
-    def execute(self, loadbalancer):
+    def execute(self, loadbalancer, lb_count_subnet):
         """Allocate a vip to the loadbalancer."""
 
         LOG.debug("Allocate_vip port_id %s, subnet_id %s,"
@@ -442,7 +456,7 @@ class AllocateVIP(BaseNetworkTask):
                   loadbalancer.vip.ip_address)
         return self.network_driver.allocate_vip(loadbalancer)
 
-    def revert(self, result, loadbalancer, *args, **kwargs):
+    def revert(self, result, loadbalancer, lb_count_subnet, *args, **kwargs):
         """Handle a failure to allocate vip."""
 
         if isinstance(result, failure.Failure):
@@ -451,7 +465,7 @@ class AllocateVIP(BaseNetworkTask):
         vip = result
         LOG.warning("Deallocating vip %s", vip.ip_address)
         try:
-            self.network_driver.deallocate_vip(vip)
+            self.network_driver.deallocate_vip(vip, lb_count_subnet)
         except Exception as e:
             LOG.error("Failed to deallocate VIP.  Resources may still "
                       "be in use from vip: %(vip)s due to error: %(except)s",
@@ -461,19 +475,10 @@ class AllocateVIP(BaseNetworkTask):
 class DeallocateVIP(BaseNetworkTask):
     """Task to deallocate a VIP."""
 
-    def execute(self, loadbalancer):
+    def execute(self, loadbalancer, lb_count_subnet):
         """Deallocate a VIP."""
-
         LOG.debug("Deallocating a VIP %s", loadbalancer.vip.ip_address)
-
-        # NOTE(blogan): this is kind of ugly but sufficient for now.  Drivers
-        # will need access to the load balancer that the vip is/was attached
-        # to.  However the data model serialization for the vip does not give a
-        # backref to the loadbalancer if accessed through the loadbalancer.
-        vip = loadbalancer.vip
-        vip.load_balancer = loadbalancer
-        self.network_driver.deallocate_vip(vip)
-        return
+        self.network_driver.deallocate_vip(loadbalancer, lb_count_subnet)
 
 
 class UpdateVIP(BaseNetworkTask):
@@ -481,7 +486,6 @@ class UpdateVIP(BaseNetworkTask):
 
     def execute(self, loadbalancer):
         LOG.debug("Updating VIP of load_balancer %s.", loadbalancer.id)
-
         self.network_driver.update_vip(loadbalancer)
 
 
@@ -679,123 +683,123 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
         self.added_fip_ports = []
         super(HandleVRIDFloatingIP, self).__init__(*arg, **kwargs)
 
+    def _add_vrid_to_list(self, vrid_list, subnet, project_id):
+        vrid_value = CONF.a10_global.vrid
+        filtered_vrid_list = list(filter(lambda x: x.subnet_id == subnet.id, vrid_list))
+        if not filtered_vrid_list:
+            vrid_list.append(
+                data_models.VRID(
+                    id=uuidutils.generate_uuid(),
+                    vrid=vrid_value,
+                    project_id=project_id,
+                    vrid_port_id=None,
+                    vrid_floating_ip=None,
+                    subnet_id=subnet.id))
+        return vrid_list
+
+    def _remove_device_vrid_fip(self, partition_name, vrid_value):
+        try:
+            is_partition = (partition_name is not None and partition_name != 'shared')
+            self.axapi_client.vrrpa.update(vrid_value, floating_ips=[], is_partition=is_partition)
+        except (acos_errors.ACOSException, req_exceptions.ConnectionError) as e:
+            LOG.exception("Failed to remove VRRP floating IPs for vrid: %s",
+                          str(vrid_value))
+            raise e
+
+    def _update_device_vrid_fip(self, partition_name, vrid_floating_ip_list, vrid_value):
+        try:
+            is_partition = (partition_name is not None and partition_name != 'shared')
+            self.axapi_client.vrrpa.update(
+                vrid_value, floating_ips=vrid_floating_ip_list, is_partition=is_partition)
+        except (acos_errors.ACOSException, req_exceptions.ConnectionError) as e:
+            LOG.exception("Failed to update VRRP floating IP %s for vrid: %s",
+                          vrid_floating_ip_list, str(vrid_value))
+            raise e
+
+    def _delete_vrid_port(self, vrid_port_id):
+        try:
+            self.network_driver.delete_port(vrid_port_id)
+        except Exception as e:
+            LOG.error("Failed to delete neutron port for VRID port: %s",
+                      vrid_port_id)
+            raise e
+
+    def _replace_vrid_port(self, vrid, subnet, lb_resource, conf_floating_ip=None):
+        if vrid.vrid_port_id:
+            self._delete_vrid_port(vrid.vrid_port_id)
+
+        try:
+            amphorae = a10_task_utils.attribute_search(lb_resource, 'amphorae')
+            fip_obj = self.network_driver.allocate_vrid_fip(
+                vrid, subnet.network_id, amphorae,
+                fixed_ip=conf_floating_ip)
+            vrid.vrid_port_id = fip_obj.id
+            vrid.vrid_floating_ip = fip_obj.fixed_ips[0].ip_address
+            self.added_fip_ports.append(fip_obj)
+        except Exception as e:
+            msg = "Failed to create neutron port for SLB resource: %s "
+            if conf_floating_ip:
+                msg += "with floating IP {}".format(conf_floating_ip)
+            LOG.error(msg, lb_resource.id)
+            raise e
+        return vrid
+
     @axapi_client_decorator
     def execute(self, vthunder, lb_resource, vrid_list, subnet):
         """
-
         :param vthunder:
         :param lb_resource: Can accept LB or member
         :param vrid_list: VRID object list for LB resource's project.
         :param subnet: subnet of the resource in question, will be helpful if there is no
         VRID object present for the provided subnet then is should create new VRID
         floating IP instead of updating existing(delete + create -> update)
+
         :return: return the update list of VRID object, If empty the need to remove all VRID
         objects from DB else need update existing ones.
         """
+
+        vrid_value = CONF.a10_global.vrid
+        prev_vrid_value = vrid_list[0].vrid if vrid_list else None
+        conf_floating_ip = a10_utils.get_vrid_floating_ip_for_project(lb_resource.project_id)
+
+        if not conf_floating_ip:
+            for vrid in vrid_list:
+                self._delete_vrid_port(vrid.vrid_port_id)
+            vrid_value = prev_vrid_value if prev_vrid_value else vrid_value
+            self._remove_device_vrid_fip(vthunder.partition_name, vrid_value)
+            return []
+
         vrid_floating_ips = []
         update_vrid_flag = False
-        vrid_value = CONF.a10_global.vrid
-        conf_floating_ip = a10_utils.get_vrid_floating_ip_for_project(
-            lb_resource.project_id)
-        prev_vrid_value = copy.deepcopy(
-            vrid_list[0].vrid) if vrid_list else None
-
-        if conf_floating_ip:
-            for vr in vrid_list:
-                if vr.subnet_id == subnet.id:
-                    break
-            else:
-                vrid_list.append(
-                    data_models.VRID(
-                        vrid=vrid_value,
-                        project_id=lb_resource.project_id,
-                        vrid_port_id=None,
-                        vrid_floating_ip=None,
-                        subnet_id=subnet.id))
+        vrid_list = self._add_vrid_to_list(vrid_list, subnet, lb_resource.project_id)
+        for vrid in vrid_list:
+            subnet = self.network_driver.get_subnet(vrid.subnet_id)
+            vrid.vrid = vrid_value
             if conf_floating_ip.lower() == 'dhcp':
-                for vrid in vrid_list:
-                    subnet = self.network_driver.get_subnet(vrid.subnet_id)
-                    subnet_ip, subnet_mask = a10_utils.get_net_info_from_cidr(
-                        subnet.cidr)
-                    vrid.vrid = vrid_value
-                    if not a10_utils.check_ip_in_subnet_range(
-                            vrid.vrid_floating_ip, subnet_ip, subnet_mask):
-                        try:
-                            # delete existing port associated to vrid in
-                            # question.
-                            if vrid.vrid_port_id:
-                                self.network_driver.delete_port(
-                                    vrid.vrid_port_id)
-                            fip_obj = self.network_driver.create_port(
-                                subnet.network_id, subnet.id)
-                            self.added_fip_ports.append(fip_obj)
-                            vrid.vrid_floating_ip = fip_obj.fixed_ips[0].ip_address
-                            vrid.vrid_port_id = fip_obj.id
-                            update_vrid_flag = True
-                        except Exception as e:
-                            LOG.error(
-                                "Failed to create neutron port for lb_resource: %s",
-                                lb_resource.id)
-                            raise e
-                    vrid_floating_ips.append(vrid.vrid_floating_ip)
+                subnet_ip, subnet_mask = a10_utils.get_net_info_from_cidr(
+                    subnet.cidr)
+                if not a10_utils.check_ip_in_subnet_range(
+                        vrid.vrid_floating_ip, subnet_ip, subnet_mask):
+                    vrid = self._replace_vrid_port(vrid, subnet, lb_resource)
+                    update_vrid_flag = True
             else:
-                for vrid in vrid_list:
-                    subnet = self.network_driver.get_subnet(vrid.subnet_id)
-                    conf_floating_ip = a10_utils.get_vrid_floating_ip_for_project(
-                        lb_resource.project_id)
-                    conf_floating_ip = a10_utils.get_patched_ip_address(
-                        conf_floating_ip, subnet.cidr)
-                    vrid.vrid = vrid_value
-                    if conf_floating_ip != vrid.vrid_floating_ip:
-                        try:
-                            # delete existing port associated to vrid in
-                            # question.
-                            if vrid.vrid_port_id:
-                                self.network_driver.delete_port(
-                                    vrid.vrid_port_id)
-                            fip_obj = self.network_driver.create_port(
-                                subnet.network_id, subnet.id, fixed_ip=conf_floating_ip)
-                            self.added_fip_ports.append(fip_obj)
-                            vrid.vrid_floating_ip = fip_obj.fixed_ips[0].ip_address
-                            vrid.vrid_port_id = fip_obj.id
-                            update_vrid_flag = True
-                        except Exception as e:
-                            LOG.error(
-                                "Failed to create neutron port for loadbalancer resource: %s with "
-                                "floating IP %s", lb_resource.id, conf_floating_ip)
-                            raise e
-                    vrid_floating_ips.append(vrid.vrid_floating_ip)
-        else:
-            for vrid in vrid_list:
-                try:
-                    self.network_driver.delete_port(vrid.vrid_port_id)
-                except Exception as e:
-                    LOG.error(
-                        "Failed to delete neutron port for VRID FIP: %s",
-                        vrid.vrid_floating_ip)
-                    raise e
-                update_vrid_flag = True
-            vrid_list = []
+                new_ip = a10_utils.get_patched_ip_address(
+                    conf_floating_ip, subnet.cidr)
+                if new_ip != vrid.vrid_floating_ip:
+                    vrid = self._replace_vrid_port(vrid, subnet, lb_resource, new_ip)
+                    update_vrid_flag = True
+            vrid_floating_ips.append(vrid.vrid_floating_ip)
+
         if (prev_vrid_value is not None) and (prev_vrid_value != vrid_value):
-            self.update_device_vrid_fip(vthunder, [], prev_vrid_value)
-            self.update_device_vrid_fip(
-                vthunder, vrid_floating_ips, vrid_value)
+            self._remove_device_vrid_fip(vthunder.partition_name, prev_vrid_value)
+            self._update_device_vrid_fip(vthunder.partition_name, vrid_floating_ips, vrid_value)
         elif update_vrid_flag:
-            self.update_device_vrid_fip(
-                vthunder, vrid_floating_ips, vrid_value)
+            self._update_device_vrid_fip(vthunder.partition_name, vrid_floating_ips, vrid_value)
+
         return vrid_list
 
     @axapi_client_decorator
-    def revert(
-            self,
-            result,
-            vthunder,
-            lb_resource,
-            vrid_list,
-            subnet,
-            *args,
-            **kwargs):
-
+    def revert(self, result, vthunder, lb_resource, vrid_list, subnet, *args, **kwargs):
         LOG.warning(
             "Reverting VRRP floating IP delta task for lb_resource %s",
             lb_resource.id)
@@ -809,44 +813,28 @@ class HandleVRIDFloatingIP(BaseNetworkTask):
                     port.id,
                     str(e))
 
-        # Normalize old vrid entries
         vrid_floating_ip_list = [vrid.vrid_floating_ip for vrid in vrid_list]
+
         if vrid_floating_ip_list:
             vrid_value = CONF.a10_global.vrid
             try:
-                self.update_device_vrid_fip(
+                self._update_device_vrid_fip(
                     vthunder, vrid_floating_ip_list, vrid_value)
             except Exception as e:
                 LOG.error("Failed to update VRID floating IPs %s due to %s",
                           vrid_floating_ip_list, str(e))
 
-    def update_device_vrid_fip(
-            self,
-            vthunder,
-            vrid_floating_ip_list,
-            vrid_value):
-        try:
-            if not vthunder.partition_name or vthunder.partition_name == 'shared':
-                self.axapi_client.vrrpa.update(
-                    vrid_value, floating_ips=vrid_floating_ip_list)
-            else:
-                self.axapi_client.vrrpa.update(
-                    vrid_value, floating_ips=vrid_floating_ip_list, is_partition=True)
-        except (acos_errors.ACOSException, req_exceptions.ConnectionError) as e:
-            LOG.exception("Failed to update VRRP floating IP %s for vrid: %s",
-                          vrid_floating_ip_list, str(vrid_value))
-            raise e
-
 
 class DeleteVRIDPort(BaseNetworkTask):
 
     """Delete VRID Port if the last resource associated with it is deleted"""
 
     @axapi_client_decorator
-    def execute(self, vthunder, vrid_list, subnet, lb_count, member_count):
+    def execute(self, vthunder, vrid_list, subnet,
+                lb_count_subnet, member_count, lb_resource):
         vrid = None
         vrid_floating_ip_list = []
-        resource_count = lb_count + member_count
+        resource_count = lb_count_subnet + member_count
         if resource_count <= 1 and vthunder:
             for vr in vrid_list:
                 if vr.subnet_id == subnet.id:
@@ -855,7 +843,8 @@ class DeleteVRIDPort(BaseNetworkTask):
                     vrid_floating_ip_list.append(vr.vrid_floating_ip)
             if vrid:
                 try:
-                    self.network_driver.delete_port(vrid.vrid_port_id)
+                    amphorae = a10_task_utils.attribute_search(lb_resource, 'amphorae')
+                    self.network_driver.deallocate_vrid_fip(vrid, subnet, amphorae)
                     if not vthunder.partition_name or vthunder.partition_name == 'shared':
                         self.axapi_client.vrrpa.update(
                             vrid.vrid, floating_ips=vrid_floating_ip_list)
@@ -875,15 +864,19 @@ class DeleteVRIDPort(BaseNetworkTask):
 
 class DeleteMultipleVRIDPort(BaseNetworkTask):
     @axapi_client_decorator
-    def execute(self, vthunder, vrid_list, subnet_list):
+    def execute(self, vthunder, vrid_list, subnet_list, lb_resource):
         try:
             if subnet_list and vthunder and vrid_list:
+                amphorae = a10_task_utils.attribute_search(lb_resource, 'amphorae')
                 vrids = []
                 vrid_floating_ip_list = []
                 for vrid in vrid_list:
-                    if vrid.subnet_id in subnet_list:
+                    subnet_matched = list(filter(lambda x: x == vrid.subnet_id,
+                                          subnet_list))
+                    if subnet_matched:
                         vrids.append(vrid)
-                        self.network_driver.delete_port(vrid.vrid_port_id)
+                        subnet = self.network_driver.get_subnet(vrid.subnet_id)
+                        self.network_driver.deallocate_vrid_fip(vrid, subnet, amphorae)
                     else:
                         vrid_floating_ip_list.append(vrid.vrid_floating_ip)
                 if not vthunder.partition_name or vthunder.partition_name == 'shared':
@@ -906,7 +899,7 @@ class GetSubnetVLANIDParent(object):
         network_id = self.network_driver.get_subnet(subnet_id).network_id
         network = self.network_driver.get_network(network_id)
         if network.provider_network_type != 'vlan':
-            raise
+            raise Exception()
         return network.provider_segmentation_id
 
 
