@@ -19,9 +19,12 @@ from oslo_log import log as logging
 from requests.exceptions import ConnectionError
 from taskflow import task
 
+from octavia.common import exceptions
+
 from a10_octavia.common import a10constants
 from a10_octavia.common import openstack_mappings
 from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator
+from a10_octavia.controller.worker.tasks.decorators import axapi_client_decorator_for_revert
 from a10_octavia.controller.worker.tasks import utils
 
 CONF = cfg.CONF
@@ -54,11 +57,19 @@ class CreateAndAssociateHealthMonitor(task.Task):
                 flavors.update(parsed_exprs)
                 args.update({'monitor': flavors})
 
+        try:
+            health_mon.type = openstack_mappings.hm_type(self.axapi_client, health_mon.type)
+        except Exception:
+            raise exceptions.ProviderUnsupportedOptionError(
+                prov="A10",
+                user_msg=("Failed to create health monitor {}, "
+                          "A health monitor of type {} is not supported "
+                          "by A10 provider").format(health_mon.id, health_mon.type))
+
         try:
             post_data = CONF.health_monitor.post_data
             self.axapi_client.slb.hm.create(health_mon.id,
-                                            openstack_mappings.hm_type(self.axapi_client,
-                                                                       health_mon.type),
+                                            health_mon.type,
                                             health_mon.delay, health_mon.timeout,
                                             health_mon.rise_threshold, method=method,
                                             port=listeners[0].protocol_port, url=url,
@@ -82,7 +93,7 @@ class CreateAndAssociateHealthMonitor(task.Task):
                 health_mon.id, health_mon.pool_id)
             raise e
 
-    @axapi_client_decorator
+    @axapi_client_decorator_for_revert
     def revert(self, listeners, health_mon, vthunder, *args, **kwargs):
         try:
             self.axapi_client.slb.hm.delete(health_mon.id)
