@@ -20,6 +20,7 @@ MOCK_LOAD_BALANCER_ID = 'mock-lb-1'
 MOCK_LISTENER_ID = 'mock-listener-1'
 MOCK_FLAVOR_ID = 'mock-flavor-1'
 MOCK_FLAVOR_PROF_ID = 'mock-flavor-prof-1'
+MOCK_AMPHORA_ID = 'amphora_id'
 MOCK_COOKIE_NAME = 'mock-cookie-1'
 MOCK_CERT_FILENAME = 'mock-cert-filename-1'
 MOCK_CERT_CONTENT = 'mock-cert-content-1'
@@ -51,12 +52,14 @@ MOCK_PROJECT_ID = 'mock-project-id'
 
 A10_GLOBAL_CONF_SECTION = 'a10_global'
 A10_HARDWARE_THUNDER_CONF_SECTION = 'hardware_thunder'
+A10_CONTROLLER_WORKER_CONF_SECTION = 'a10_controller_worker'
 LISTENER_CONF_SECTION = 'listener'
 SERVICE_GROUP_CONF_SECTION = 'service_group'
 SERVER_CONF_SECTION = 'server'
 VIRTUAL_SERVER_CONFIG_SECTION = 'slb'
 HEALTH_MONITOR_SECTION = 'health_monitor'
 
+
 A10_HOUSE_KEEPING = 'a10_house_keeping'
 
 MOCK_SERVICE_GROUP_PROTOCOL = "HTTP"
@@ -67,3 +70,4 @@ MOCK_L7POLICY_ID = "mock-l7-policy-1"
 MOCK_L7RULE_ID = "mock-l7-rule-1"
 
 MOCK_TLS_CERTIFICATE_ID = "mock_tls_certificate_id"
+MOCK_TOPOLOGY_ACTIVE_STANDBY = "ACTIVE_STANDBY"
