@@ -57,9 +57,10 @@ class TestLoadBalancerFlows(base.TestCase):
         self.conf.reset()
 
     def test_create_lb_flows(self, mock_net_driver):
-        target = self.flows.get_create_load_balancer_flow(constants.TOPOLOGY_SINGLE)
-        self.assertIsInstance(target, flow.Flow)
-        self.assertIn("vthunder", target.provides)
+        lb = o_data_models.LoadBalancer(id=a10constants.MOCK_LOAD_BALANCER_ID,
+                                        project_id='project-vthunder')
+        (create_flow, store) = self.flows.get_delete_load_balancer_flow(lb, False, False)
+        self.assertIsInstance(create_flow, flow.Flow)
 
     def test_create_lb_rack_vthunder_vlan_flow(self, mock_net_driver):
         self.conf.register_opts(config_options.A10_GLOBAL_OPTS,
@@ -79,5 +80,5 @@ class TestLoadBalancerFlows(base.TestCase):
                          devices=[RACK_DEVICE])
         lb = o_data_models.LoadBalancer(id=a10constants.MOCK_LOAD_BALANCER_ID,
                                         project_id='project-rack-vthunder')
-        (del_flow, store) = self.flows.get_delete_load_balancer_flow(lb, False)
+        (del_flow, store) = self.flows.get_delete_load_balancer_flow(lb, False, False)
         self.assertIsInstance(del_flow, flow.Flow)
