@@ -70,10 +70,11 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         return_value=None)
     def test_HandleVRIDFloatingIP_noop_vrrpa_config_not_specified(
             self, mock_utils):
-        network_task = a10_network_tasks.HandleVRIDFloatingIP()
+        mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
+        mock_network_task.axapi_client = self.client_mock
         subnet = copy.deepcopy(SUBNET_1)
         subnet.cidr = a10constants.MOCK_SUBNET_CIDR
-        result = network_task.execute(VTHUNDER, MEMBER, [], subnet)
+        result = mock_network_task.execute(VTHUNDER, MEMBER, [], subnet)
         self.assertEqual(result, [])
 
     @mock.patch('a10_octavia.common.utils.get_vrid_floating_ip_for_project',
@@ -91,23 +92,24 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(VTHUNDER, member, [], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id,
-            member.subnet_id,
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            mock.ANY, None, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FLOATING_IP_1)
         self.client_mock.vrrpa.update.assert_called_with(
-            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_1])
+            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_1],
+            is_partition=False)
 
     @mock.patch('a10_octavia.common.utils.get_vrid_floating_ip_for_project',
                 return_value=a10constants.MOCK_VRID_FLOATING_IP_1)
     @mock.patch('a10_octavia.common.utils.get_patched_ip_address',
                 return_value=a10constants.MOCK_VRID_FLOATING_IP_1)
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
     def test_HandleVRIDFloatingIP_create_floating_ip_in_specified_partition_with_static_ip(
-            self, mock_patched_ip, get_floating_ip):
+            self, mock_utils, mock_patched_ip, get_floating_ip):
         member = copy.deepcopy(MEMBER)
         member.subnet_id = SUBNET_1.id
         subnet = copy.deepcopy(SUBNET_1)
@@ -119,13 +121,12 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(vthunder, member, [], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id,
-            member.subnet_id,
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            mock.ANY, None, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FLOATING_IP_1)
         self.client_mock.vrrpa.update.assert_called_with(
             VRID_VALUE, floating_ips=[
@@ -148,14 +149,15 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(VTHUNDER, member, [VRID_1], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id, member.subnet_id)
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            mock.ANY, None, None, fixed_ip=None)
         self.client_mock.vrrpa.update.assert_called_with(
-            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_1])
+            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_1],
+            is_partition=False)
 
     @mock.patch(
         'a10_octavia.common.utils.check_ip_in_subnet_range',
@@ -176,12 +178,12 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(vthunder, member, [VRID_1], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id, member.subnet_id)
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            mock.ANY, None, None, fixed_ip=None)
         self.client_mock.vrrpa.update.assert_called_with(
             VRID_VALUE, floating_ips=[
                 a10constants.MOCK_VRID_FLOATING_IP_1], is_partition=True)
@@ -200,7 +202,7 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         self.network_driver_mock.delete_port.assert_called_with(
             vrid.vrid_port_id)
         self.client_mock.vrrpa.update.assert_called_with(
-            vrid.vrid, floating_ips=[])
+            vrid.vrid, floating_ips=[], is_partition=False)
         self.assertEqual(result, [])
 
     @mock.patch('a10_octavia.common.utils.get_vrid_floating_ip_for_project',
@@ -222,15 +224,16 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         mock_network_task.execute(VTHUNDER, member, [vrid], subnet)
-        self.network_driver_mock.create_port.assert_not_called()
+        self.network_driver_mock.allocate_vrid_fip.assert_not_called()
         self.client_mock.vrrpa.update.assert_not_called()
 
     @mock.patch('a10_octavia.common.utils.get_vrid_floating_ip_for_project',
                 return_value=a10constants.MOCK_VRID_FLOATING_IP_2)
     @mock.patch('a10_octavia.common.utils.get_patched_ip_address',
                 return_value=a10constants.MOCK_VRID_FLOATING_IP_2)
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
     def test_HandleVRIDFloatingIP_replace_floating_ip_in_shared_partition_with_static_ip(
-            self, mock_patched_ip, get_floating_ip):
+            self, mock_utils, mock_patched_ip, get_floating_ip):
         vrid = copy.deepcopy(VRID_1)
         vrid.vrid_floating_ip = a10constants.MOCK_VRID_FLOATING_IP_1
         vrid.vrid = VRID_VALUE
@@ -244,16 +247,16 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(VTHUNDER, member, [vrid], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id,
-            member.subnet_id,
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            vrid, subnet.network_id, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FLOATING_IP_2)
         self.client_mock.vrrpa.update.assert_called_with(
-            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_2])
+            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_2],
+            is_partition=False)
         self.network_driver_mock.delete_port.assert_called_with(
             a10constants.MOCK_VRRP_PORT_ID)
 
@@ -261,8 +264,9 @@ class TestNetworkTasks(base.BaseTaskTestCase):
                 return_value=a10constants.MOCK_VRID_FLOATING_IP_2)
     @mock.patch('a10_octavia.common.utils.get_patched_ip_address',
                 return_value=a10constants.MOCK_VRID_FLOATING_IP_2)
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
     def test_HandleVRIDFloatingIP_replace_floating_ip_in_specified_partition_with_static_ip(
-            self, mock_patched_ip, get_floating_ip):
+            self, mock_utils, mock_patched_ip, get_floating_ip):
         vrid = copy.deepcopy(VRID_1)
         vrid.vrid_floating_ip = a10constants.MOCK_VRID_FLOATING_IP_1
         vrid.vrid = VRID_VALUE
@@ -278,13 +282,12 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(vthunder, member, [vrid], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id,
-            member.subnet_id,
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            vrid, None, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FLOATING_IP_2)
         self.client_mock.vrrpa.update.assert_called_with(
             VRID_VALUE, floating_ips=[
@@ -310,7 +313,7 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
         fip_port = mock_network_task.execute(VTHUNDER, member, [vrid], subnet)
-        self.network_driver_mock.create_port.assert_not_called()
+        self.network_driver_mock.allocate_vrid_fip.assert_not_called()
         self.network_driver_mock.delete_port.assert_not_called()
         self.assertEqual(fip_port, [vrid])
 
@@ -320,8 +323,9 @@ class TestNetworkTasks(base.BaseTaskTestCase):
     @mock.patch(
         'a10_octavia.common.utils.get_vrid_floating_ip_for_project',
         return_value='dhcp')
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
     def test_HandleVRIDFloatingIP_replace_floating_ip_diff_subnet_in_shared_part_conf_fip_set_dhcp(
-            self, get_floating_ip, check_subnet):
+            self, mock_utils, get_floating_ip, check_subnet):
         vrid = copy.deepcopy(VRID_1)
         vrid.vrid_floating_ip = a10constants.MOCK_VRID_FLOATING_IP_1
         vrid.vrid = VRID_VALUE
@@ -336,25 +340,25 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(VTHUNDER, member, [vrid], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id, member.subnet_id)
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            vrid, subnet.network_id, mock.ANY, fixed_ip=None)
         self.client_mock.vrrpa.update.assert_called_with(
-            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_1])
+            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FLOATING_IP_1],
+            is_partition=False)
         self.network_driver_mock.delete_port.assert_called_with(
             a10constants.MOCK_VRRP_PORT_ID)
 
-    @mock.patch(
-        'a10_octavia.common.utils.check_ip_in_subnet_range',
-        return_value=False)
-    @mock.patch(
-        'a10_octavia.common.utils.get_vrid_floating_ip_for_project',
-        return_value='dhcp')
+    @mock.patch('a10_octavia.common.utils.check_ip_in_subnet_range',
+                return_value=False)
+    @mock.patch('a10_octavia.common.utils.get_vrid_floating_ip_for_project',
+                return_value='dhcp')
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
     def test_HandleVRIDFloatingIP_replace_floating_ip_diff_subnet_in_set_part_conf_fip_set_dhcp(
-            self, get_floating_ip, check_subnet):
+            self, mock_utils, get_floating_ip, check_subnet):
         vrid = copy.deepcopy(VRID_1)
         vrid.vrid_floating_ip = a10constants.MOCK_VRID_FLOATING_IP_1
         vrid.vrid = VRID_VALUE
@@ -370,12 +374,12 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(vthunder, member, [vrid], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id, member.subnet_id)
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            vrid, subnet.network_id, mock.ANY, fixed_ip=None)
         self.client_mock.vrrpa.update.assert_called_with(
             VRID_VALUE, floating_ips=[
                 a10constants.MOCK_VRID_FLOATING_IP_1], is_partition=True)
@@ -384,8 +388,9 @@ class TestNetworkTasks(base.BaseTaskTestCase):
 
     @mock.patch('a10_octavia.common.utils.get_vrid_floating_ip_for_project',
                 return_value=a10constants.MOCK_VRID_PARTIAL_FLOATING_IP)
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
     def test_HandleVRIDFloatingIP_creating_floating_ip_conf_fip_is_partial(
-            self, get_floating_ip):
+            self, get_floating_ip, mock_utils):
         member = copy.deepcopy(MEMBER)
         member.subnet_id = a10constants.MOCK_SUBNET_ID
         subnet = copy.deepcopy(SUBNET_1)
@@ -396,26 +401,28 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         mock_network_task = a10_network_tasks.HandleVRIDFloatingIP()
         mock_network_task.axapi_client = self.client_mock
         self.network_driver_mock.get_subnet.return_value = subnet
-        self.network_driver_mock.create_port.return_value = port
+        self.network_driver_mock.allocate_vrid_fip.return_value = port
         self.conf.config(group=a10constants.A10_GLOBAL_OPTS,
                          vrid=VRID_VALUE)
         mock_network_task.execute(VTHUNDER, member, [VRID_1], subnet)
-        self.network_driver_mock.create_port.assert_called_with(
-            subnet.network_id,
-            member.subnet_id,
+        self.network_driver_mock.allocate_vrid_fip.assert_called_with(
+            mock.ANY, subnet.network_id, mock.ANY,
             fixed_ip=a10constants.MOCK_VRID_FULL_FLOATING_IP)
         self.client_mock.vrrpa.update.assert_called_with(
-            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FULL_FLOATING_IP])
+            VRID_VALUE, floating_ips=[a10constants.MOCK_VRID_FULL_FLOATING_IP],
+            is_partition=False)
 
-    def test_DeleteMemberVRIDPort_delete_vrid_ip_member_count_equals_one(self):
+    @mock.patch('a10_octavia.controller.worker.tasks.a10_network_tasks.a10_task_utils')
+    def test_DeleteMemberVRIDPort_delete_vrid_ip_member_count_equals_one(self, mock_utils):
         mock_network_task = a10_network_tasks.DeleteVRIDPort()
         vrid = copy.deepcopy(VRID_1)
         vrid.vrid_port_id = a10constants.MOCK_VRRP_PORT_ID
         vrid.vrid = VRID_VALUE
         mock_network_task.axapi_client = self.client_mock
-        result = mock_network_task.execute(VTHUNDER, [vrid], SUBNET_1, 0, 1)
-        self.network_driver_mock.delete_port.assert_called_with(
-            a10constants.MOCK_VRRP_PORT_ID)
+        result = mock_network_task.execute(VTHUNDER, [vrid], SUBNET_1,
+                                           0, 1, MEMBER)
+        self.network_driver_mock.deallocate_vrid_fip.assert_called_with(
+            vrid, mock.ANY, mock.ANY)
         self.client_mock.vrrpa.update.assert_called_with(
             vrid.vrid, floating_ips=[])
         self.assertEqual(result, (vrid, True))
@@ -423,8 +430,9 @@ class TestNetworkTasks(base.BaseTaskTestCase):
     def test_DeleteMemberVRIDPort_member_count_lb_count(self):
         mock_network_task = a10_network_tasks.DeleteVRIDPort()
         mock_network_task.axapi_client = self.client_mock
-        result = mock_network_task.execute(VTHUNDER, [VRID_1], SUBNET_1, 1, 1)
-        self.network_driver_mock.delete_port.assert_not_called()
+        result = mock_network_task.execute(VTHUNDER, [VRID_1], SUBNET_1,
+                                           1, 1, MEMBER)
+        self.network_driver_mock.deallocate_vrid_fip.assert_not_called()
         self.client_mock.vrrpa.delete.assert_not_called()
         self.assertEqual(result, (None, False))
 
@@ -438,7 +446,7 @@ class TestNetworkTasks(base.BaseTaskTestCase):
         self.network_driver_mock.delete_port.assert_called_with(
             a10constants.MOCK_VRRP_PORT_ID)
         self.client_mock.vrrpa.update.assert_called_with(
-            vrid.vrid, floating_ips=[])
+            vrid.vrid, floating_ips=[], is_partiton=False)
         self.assertEqual(result, [])
 
     def delete_multi_vrid_port_against_all_subnet_used(self):
